from pydantic import BaseModel, Field, ConfigDict
from typing import List, Optional
from JMTrucoCmd.models.type_card import TypeCard
class CardBase(BaseModel):
    number: int
    type_cart : Optional[TypeCard] = Field(None)
    #model config
    model_config = ConfigDict(arbitrary_types_allowed=True)

class CardInit(CardBase):
    pass
    





from random import randint
#models
from JMTrucoCmd.models.stats import Stats
#schemas
from JMTrucoCmd.schemas.bot import BotInit
from JMTrucoCmd.schemas.stats import StatsInit
#functions
from JMTrucoCmd.functions.deck import calculate_envido,calculate_truco, calculate_win, get_deck

class Bot():
    def __init__(self,bot_init:BotInit) -> None:
        self.name:str = (bot_init.name or 'Pc').capitalize()
        self.stats:Stats = Stats(
            StatsInit(
                aggressive=bot_init.aggressive or randint(1,10),
                liar=bot_init.liar or randint(1,10),
                fisherman=bot_init.fisherman or randint(1,10),
                score_truco=bot_init.score_truco or randint(8,9),
                score_envido=bot_init.score_envido or randint(23,27)
            )
        )
        self.phrases:dict = bot_init.phrases or {
            "Truco": [
                [
                "No seas un cuatro de copas y aceptame un",
                "¡Truco!"
                ],
                "¡Truco!",
                "¿Te animás al Truco? Vamos a ver qué tenés.",
                [
                    "El Truco está en jugar bien tus cartas,",
                    "¡a ver si te animás!"
                ],
                "¡A ver qué tenés ahí, eh! ¡Truco, loco!",
                [
                    "Ahora sí que se pone picante esto",
                    "¡Truco!"
                ],
                [
                    'Tengo apuro por ganar',
                    "y no quiero padecer",
                    'Truco te voy a cantar',
                    'Para poderte vencer',
                ],
                [
                    "Truco...",
                    "este eh...",
                    "...",
                    "Bueno, ¡Digo Truco!"
                ],
            ],
            "Retruco": [
                [
                    "No te achiques",
                    '¡Retruco!'
                ],
                [
                    "¡Naah, eso no me asusta!",
                    "¡Retruco, dale!"
                ],
                '¡Retruco!',
                "¿Queres Retruco? Asi se pone interesante",
                "El Retruco es para valientes, ¿te la jugás?",
                "Amargo y Retruco"
            ],
            "Vale Cuatro": [
                [
                    "¡Vale cuatro!",
                    "Este es el momento de jugársela toda."
                ],
                [
                    "Todo o nada...",
                    "Vale Cuatro!"
                    "¿te animás?",
                ],
                "El Vale Cuatro es para los que no tienen miedo, cagon",
                "¡Vale Cuatro!"
            ],
            "Envido": [
                [
                    "Cuando vine de La Isla",
                    "encontre un lazo retorcido",
                    "con él enlacé dos cartas",
                    "y con dos le digo Envido"

                ],
                [
                   "La mejor mano gana",
                   "¡Envido!",
                ],
                "¡Envido!",
                "El envido se gana con buen puntaje, ¿cuánto tenés?",
                [
                    "A este humano reprimido",
                    "le voy a cantar el Envido",
                ],
                [
                    "Aunque yo tenga pinta de computadora",
                    "hay un enano aqui metido",
                    "que me esta' dele soplar",
                    '"Dale boludo, que tenes Envido"'
                ],
            ],
            "Real Envido": [
                "¡Real envido!",
                [
                   " Con su boquita de grana",
                    "y su pelo renegrido",
                    "no envidia a la mañana",
                    "este hermoso Real envido"
                ],
                "Un Real Envido por favor",
                "La apuesta sube... ¡Real envido! ",
                "Real envido, vamos a ver quién tiene más.",
                "El Real Envido es para subirle?",
                [
                    "A este humano reprimido",
                    'lo asusto con un Real Envido',
                ],
            ],
            "Falta Envido": [
                "¡Falta envido! Acá se define todo, ¿te animás?",
                [   
                    "Estas cansado?",
                    "Estas molido?",
                    "Entonces refrescate con dos Reales Envido"
                ],
                [
                    "Pido gancho",
                    "gancho pido",
                    "Chupate esta",
                    "Falta Envido!"
                ],
                [
                    "Che, humano",
                    "no sea tan altivo",
                    "que yo lo desafio",
                    "con una Falta Envido"
                ],
                "Falta envido, todo o nada, ¿qué decís?",
                "El falta envido define el juego, ¡a ver qué tenés!"
            ],
            "Quiero": [
                "Quiero, a ver que tengo chances",
                [
                    "¡Sí, acepto el desafío!",
                    "Quiero"
                ],
                "Quiero, vamos a ver qué pasa",
                [
                    "Mmmm",
                    "No se...",
                    "¡Quiero!"
                ]
            ],
            "No Quiero": [
                "No Quiero, mejor me guardo",
                "No Quiero, no me siento seguro",
                "No Quiero...",
                [
                    "Mmmm",
                    "No se...",
                    "No Quiero"
                ],
                [
                    "Hay veces que hay que querer",
                    "otras veces duplicar",
                    "pero hoy voy a rechazar",
                    "No Quiero"
                ]
            ],
            "Victoria":[
                [
                    "¡Vamos carajo!",
                    f"Nadie confiaba en {self.name}"
                ],
                [
                    "¡Sí, papá!",
                    'Siiiii'
                ],
                "Ez",
                "¿Donde se le sube la dificultad?",
                [
                    "Muchachos!!!",
                    "Ahora nos volvimo' a ilusionar!"
                ]
            ],
            "Derrota":[
                "Gg no team",
                "¡No pegue una, loco!",
                "Robo Robo!!!",
                "Derrota que duele en el alma..."
            ]
        }
    def __str__(self) -> str:
        return f'nombre: {self.name}, stats: ({self.stats})'
    
    def play_round(self,round:int=None,cards:list=[],cards_played:list=[],envido=1,truco = 1,quiero_pc = True):
        if (round not in [1,2,3]):
            round = 1
        
        if round == 1:
            if envido==1:
                call_envido = self.probabily_call_envido(cards)
                if call_envido is not 0:
                    return None, call_envido, None
                    pass
        if truco!=4 and quiero_pc is not False:
            call_truco = self.probabily_call_truco(cards,cards_played,truco,round)
            if call_truco is not False:
                return None, None, call_truco
                pass
        
        card = self.pick_card(cards,cards_played,round)
        return card,None,None            

                    
    def probabily_call_envido(self,cards):
        #0:ya jugado, 1:nada, 2:envido, 3:real envido, 4: falta envido 
        actual_points_envido = calculate_envido(cards)
        if actual_points_envido>=self.stats.score_envido: #tiene puntos
            #probabilidad de ir a la pesca
            if(randint(0,80) <= randint(0,self.stats.fisherman*10-self.stats.aggressive)):
                return False
            #probabilidad de que cantar
            n = 10-randint(1, self.stats.aggressive)
            if actual_points_envido >= self.stats.score_envido+n:
                if randint(0,2) <= 1:
                    return 4
            if actual_points_envido >= self.stats.score_envido+n//2:
                if randint(0,2) <= 1:
                    return 3
            return 2
        #no tiene puntos
        if randint(0,self.stats.liar*10) >= randint(0,100):
            n = randint(1, (self.stats.aggressive+self.stats.liar)//2)
            if n>=5:
                return 4
            if n>=3:
                return 3
            return 2
        return 0

    def probabily_call_truco(self,cards,cards_played,truco,round):
        #self.truco = 1  #0:,          1:nada, 2:truco,  3:retruco,     4:vale cuatro
        actual_points_truco = calculate_truco(cards)
        win_rounds, card_to_responses = calculate_win(cards_played)
        extra = 0
        if win_rounds['player'] > win_rounds['pc']:
            extra = -3
        elif win_rounds['player'] < win_rounds['pc']:
            extra = 2
        
        
        if not card_to_responses:
            pass
        else:
            for card in cards:
                if card.value>card_to_responses.value:
                    if card_to_responses.value<self.stats.score_truco+1:
                        extra+=1
        n = (extra+self.stats.score_truco)*len(cards) + randint(-2,2)

        if win_rounds['draw'] <= 1:
            if card_to_responses:
                for card in cards:
                    if card.value<card_to_responses.value:
                        return truco+1
                        
        good_carts = actual_points_truco< n
        aggresive = randint(0,100-len(cards)*20) <= randint(0,10+(self.stats.aggressive*10)) 
        liar = randint(0,80) <= randint(0,self.stats.liar*10)
        #print(f'cartas: {actual_points_truco} < expectativas: {self.stats.score_truco*len(cards)} or {n} ')
        #print(f'aggresive:{aggresive}, good carts:{good_carts}, liar: {liar}')
        
        if good_carts:      #tiene buenas cartas
            if(aggresive):
                return truco+1
        else:               #no tiene buenas cartas
            if(liar):
                return truco+1
        return False
    
    def pick_card(self,cards,cards_played,round):
        win_rounds, card_to_responses = calculate_win(cards_played)
        cards_sorted_value = self.value_card(cards)
        n = randint(1,100)
        aggresive = randint(0,80) <= randint(0,self.stats.aggressive*10)
        if win_rounds['draw']:
            return cards_sorted_value[0]
        elif win_rounds['pc'] > win_rounds['player']:
            return cards_sorted_value[-1]
        elif win_rounds['pc'] < win_rounds['player']:
            if not card_to_responses:
                if n<=self.stats.aggressive//2+self.stats.liar+5:
                    return cards_sorted_value[1]
                else:
                    return cards_sorted_value[0]
            else:
                min = self.min_winner_value_card(cards_sorted_value,card_to_responses.value)
                if min:
                    return min
        if round == 1:
            if not card_to_responses:
                if aggresive:
                    if n<=5:
                        return cards_sorted_value[0]
                    else:
                        return cards_sorted_value[1]
                if n<50:
                    return cards_sorted_value[1]
                return cards_sorted_value[2]
            else:
                if n<=self.stats.liar+5:
                    return cards_sorted_value[-1]
                min = self.min_winner_value_card(cards_sorted_value,card_to_responses.value)
                if min:
                    return min
        return cards_sorted_value[0]
    
    def value_card(self,cards):
        cards_sorted = sorted(cards, key=lambda card: card.value)
        return cards_sorted

    def min_winner_value_card(self,cards, value):
        min = None
        for i in cards:
            if value>i.value:
                if not min:
                    min = i
                if min.value < i.value:
                    min = i
        return min
    
    def respond_envido(self,cards,at_least=2):
        #0:ya jugado, 1:nada, 2:envido, 3:real envido, 4: falta envido 
        actual_points_envido = calculate_envido(cards)
        if actual_points_envido>=self.stats.score_envido+at_least+randint(-1,1): #tiene puntos
            #probabilidad de que cantar
            n = 10-randint(1, self.stats.aggressive)
            if actual_points_envido >= self.stats.score_envido+n:
                if randint(0,2) <= 1:
                    return max(at_least,4)
            if actual_points_envido >= self.stats.score_envido+n//2:
                if randint(0,2) <= 1:
                    return max(at_least,3)
            return max(at_least,2)
        #no tiene puntos
        if randint(0,self.stats.liar*10) >= randint(0,100) and at_least!=4:

            liar = randint(1, (self.stats.aggressive+self.stats.liar)//2)
            if liar>=5:
                return max(4,at_least)
            if liar>=3:
                return max(3,at_least)
            return max(2,at_least)
        return 0

    def respond_truco(self,cards,cards_played,truco):
        #self.truco = 1  #0:,          1:nada, 2:truco,  3:retruco,     4:vale cuatro
        actual_points_truco = calculate_truco(cards)
        win_rounds, card_to_responses = calculate_win(cards_played)

        extra = -truco
        if win_rounds['player'] > win_rounds['pc']:
            extra = -3
        elif win_rounds['player'] < win_rounds['pc']:
            extra = 2
        elif win_rounds['draw'] !=0:
            extra = -1
        
        
        if not card_to_responses:
            n = (extra+self.stats.score_truco)*(len(cards)+1) + randint(0,self.stats.aggressive//2)
            if len(cards_played['pc']) > len(cards_played['player']):
                actual_points_truco+=cards_played['pc'][-1].value
                if cards_played['pc'][-1].value < self.stats.score_truco:
                    n = (extra+self.stats.score_truco)*(len(cards)+1) + randint(0,self.stats.aggressive)
        else:
            n = (extra+self.stats.score_truco)*len(cards) + randint(0,self.stats.aggressive//2)
            can_defeated = False
            for card in cards:
                if card.value>card_to_responses.value:
                    n+=2
                else:
                    n+=-1
                    can_defeated = True
            if not can_defeated and (win_rounds['player'] == 1 or win_rounds['draw'] == 1):
                if card_to_responses.value == 1:
                    return None
                n = 0
        
        good_carts = actual_points_truco<= n 
        aggresive = randint(0,80) <= randint(0,10+(self.stats.aggressive*10)+len(cards)*10) 
        liar = randint(0,80) <= randint(0,self.stats.liar*10)
        #print(f'cartas: {actual_points_truco} < expectativas: {self.stats.score_truco*len(cards)} or {n} ')
        #print(f'aggresive:{aggresive}, good carts:{good_carts}, liar: {liar}')
        if good_carts:      #tiene buenas cartas
            if(aggresive) and truco!=4:
                return True
            return False
        else:               #no tiene buenas cartas
            if(liar) and truco!=4:
                if aggresive and randint(1,10) < 8 and truco!=4:
                    return True
                return False
        return None



from JMTrucoCmd.schemas.card import CardInit

class Card():
    def __init__(self,card_init:CardInit, valid_numbers = [1,2,3,4,5,6,7,10,11,12]) -> None:
        
        if card_init.number in valid_numbers:
            self.number:int = card_init.number
        else: 
            self.number:int = valid_numbers[0]
        
        self.type_card = card_init.type_cart
        self.value = self.get_value()

    def __str__(self) -> str:
        return f'{self.number} {self.type_card}'
    
    
    def get(self,completo=True) -> str:
        return f'{self.number} {self.type_card.get(completo=completo)}'
    
    def get_value(self):
        values_normal = {
            3: 5,
            2: 6,
            12: 8,
            11: 9,
            10: 10,
            6: 12,
            5: 13,
            4: 14
        }
        if self.number in values_normal:
            return values_normal[self.number]
        elif self.number == 1:
            values_1 = {
                'Espada':1,
                'Basto':2,
                'Oro':7,
                'Copa':7,
            }
            if self.type_card.name in values_1:
                return values_1[self.type_card.name]
        elif self.number == 7:
            values_7 = {
                'Espada':3,
                'Basto':11,
                'Oro':4,
                'Copa':11,
            }
            if self.type_card.name in values_7:
                return values_7[self.type_card.name]
        return 0

    def get_envido(self):
        if self.number in [10,11,12]:
            return 0
        return self.number


from pydantic import BaseModel, Field, ConfigDict
from typing import List, Optional
from JMTrucoCmd.models.stats import Stats
from JMTrucoCmd.schemas.stats import StatsInit


class BotBase(BaseModel):
    name: str
    stats:Stats

    #model config
    model_config = ConfigDict(arbitrary_types_allowed=True)

class BotInit(StatsInit):
    name: Optional[str] = None
    phrases: Optional[dict] = None


    





from pydantic import BaseModel, Field, ConfigDict
from typing import List, Optional

class StatsBase(BaseModel):
    liar: Optional[int] = None
    fisherman: Optional[int] = None
    aggressive: Optional[int] = None
    score_truco: Optional[int] = None
    score_envido: Optional[int] = None

    #model config
    model_config = ConfigDict(arbitrary_types_allowed=True)

class StatsInit(StatsBase):
    pass
    





from random import choice
from time import sleep
#models
from JMTrucoCmd.models.type_card import TypeCard
from JMTrucoCmd.models.card import Card
#schemas
from JMTrucoCmd.schemas.type_card import TypeCardInit, TypeCardGet
from JMTrucoCmd.schemas.card import CardInit

def get_deck():
    deck = []
    type_carts:list = [
        TypeCard(
            TypeCardInit(
                name='Espada',
                description='Tipo de carta: espada',
                emoji= '🔪'
            )
        ),
        TypeCard(
            TypeCardInit(
                name='basto',
                description='Tipo de carta: basto',
                emoji = '🌳'
            )
        ),
        TypeCard(
            TypeCardInit(
                name='oro',
                description='Tipo de carta: oro',
                emoji = '🥇'
            )
        ),
        TypeCard(
            TypeCardInit(
                name='copa',
                description='Tipo de carta: copa',
                emoji = '🥂'
            )
        ),
    ]
    numbers = {
        1:1,
        2:2,
        3:3,
        4:4,
        5:5,
        6:6,
        7:7,
        8:10,
        9:11,
        0:12
    }
    cont = 0
    for i in range(1,41):
        n = i%10
        cart =Card(
            CardInit(number=numbers[n], type_cart=type_carts[cont])
        )
        deck.append(cart)
        if n==0:
            cont+=1
    return deck

def get_cards():
    deck = get_deck()
    carts = []
    for i in range(6):
        cart:Card = choice(deck)
        carts.append(cart)
        deck.remove(cart)
    return carts

def calculate_envido(cards:list):
    envido_points = 0
    no_envido = cards[-1].get_envido()
    for i in range(len(cards)-1):
        if cards[i].get_envido() > no_envido:
            no_envido = cards[i].get_envido()
        for j in range(i+1,len(cards)):
            if cards[i].type_card == cards[j].type_card:
                points = 20 + cards[i].get_envido() + cards[j].get_envido()
                if not envido_points:
                    envido_points = points
                elif envido_points < points:
                    envido_points = points
    return max(envido_points,no_envido)

def calculate_truco(cards:list):
    truco_points = 0
    for card in cards:
        truco_points+=card.value
    return truco_points

def calculate_win(cards_played:dict):
    response = {
        'pc': 0,
        'player': 0,
        'draw':0
    }
    bot_responses = False
    if (len(cards_played['player']) > len(cards_played['pc'])):
        bot_responses = cards_played['player'][-1]
    
    quantity = min(len(cards_played['pc']),len(cards_played['player']))
    
    for i in range(quantity):
        if cards_played['pc'][i].value < cards_played['player'][i].value:
            response['pc']+=1
        elif cards_played['pc'][i].value > cards_played['player'][i].value:
            response['player']+=1
        else:
            response['draw']+=1
    
    return response, bot_responses



from random import randint,choice
from time import sleep
#models
from JMTrucoCmd.models.card import Card
from JMTrucoCmd.models.bot import Bot
#schemas
from JMTrucoCmd.schemas.type_card import TypeCardInit, TypeCardGet
from JMTrucoCmd.schemas.card import CardInit
from JMTrucoCmd.schemas.bot import BotInit
#functions
from JMTrucoCmd.functions.deck import get_deck, get_cards, calculate_win, calculate_envido
from JMTrucoCmd.functions.sistem import print_slow, cls
from JMTrucoCmd.functions.bots import get_bots

class Game():
    def __init__(self, center= 49, time = 1, objective = 30) -> None:
        self.time = time
        self.objective = objective
        self.bot:Bot = None
        self.start_pc = True
        self.play_pc = self.start_pc
        self.quiero_pc = None
        self.center = center
        self.score:list[int] = [0,0] #0:gamer, 1:computer
        self.actual_score:list[int] = [0,0]
        self.envido = 1 #0:ya jugado, 1:nada, 2:envido, 3:real envido, 4:falta envido 
        self.truco = 1               #1:nada, 2:truco,  3:retruco,     4:vale cuatro
        self.cards_played = {
            1: [],
            2: [],
            3: []
        }
        cards = get_cards()
        self.cards:list[list[Card]] = [
            cards[0:3], cards[3:7] 
        ]
        self.truco_options = {
            1: 'Normal',
            2: 'Truco',
            3: 'Retruco',
            4: 'Vale Cuatro'
        }
        self.envido_options = {
            2: 'Envido',
            3: 'Real Envido',
            4: 'Falta Envido'
        }
        self.envido_option_points = {
            2: 2,
            3: 3,
            4: 0
        }

    def start(self, bot:Bot = Bot(BotInit(name='robocop',aggressive=8))):
        self.bot = bot
        self.score = [0,0]
        self.actual_score = [0,0]
        if randint(0,1) == 0:
            self.start_pc = False
        else:
            self.start_pc = True
        win_match = False
        #self.print_intro_truco(time=1)
        cls()
        print_slow('Empieza la partida de truco')
        
        #self.print_phrase(bot.phrases['Truco'])
        self.print_score()
        while not win_match:
            self.start_round()
            self.start_pc = not self.start_pc
            win_match,winner = self.win_match()
        print(f'{winner} gano!!!!')

    def start_round(self):
        self.actual_score:list[int] = [0,0] #0:gamer, 1:computer
        self.score_envido:int = 0
        self.envido:int = 1 #0:ya jugado, 1:nada, 2:envido, 3:real envido, 4: falta envido 
        self.truco:int = 1  #0:,          1:nada, 2:truco,  3:retruco,     4:vale cuatro
        self.end_round:bool = False
        self.cards_played = {
            'pc': [],
            'player': []
        }
        cards = get_cards()
        self.cards:list[list[Card]] = [
            cards[0:3], cards[3:7] 
        ]
        self.all_cards = self.cards
        self.cards_envido = [
            calculate_envido(self.cards[0]),
            calculate_envido(self.cards[1]),
        ]
        self.play_pc = self.start_pc
        self.quiero_pc = None
        self.win_pc = None
        while not self.end_round:
            round = min(len(self.cards_played['pc']),len(self.cards_played['player']))+1
            cls()
            print(f'ROUND: {round} - BOT: {self.bot}')
            self.print_played_cards()
            
            if len(self.cards_played['pc']) == len(self.cards_played['player']) and round!=1:
                if self.cards_played['pc'][-1].value < self.cards_played['player'][-1].value:
                    self.play_pc = True
                elif self.cards_played['pc'][-1].value > self.cards_played['player'][-1].value:
                    self.play_pc = False
                    
            if self.play_pc:
                print('cartas disponibles'.center( self.center))
                self.print_card(cards = self.cards[0],center_left=False) 
                sleep(self.time)
                card_pc,envido,truco = self.bot.play_round(round=round,cards=self.cards[1],cards_played=self.cards_played,envido=self.envido,truco=self.truco,quiero_pc=self.quiero_pc)
                if card_pc:
                    self.cards_played['pc'].append(card_pc)
                    self.cards[1].remove(card_pc)
                    self.play_pc = not self.play_pc
                elif envido:
                    sleep(self.time)
                    self.respond_envido(envido)
                elif truco:
                    sleep(self.time)
                    self.respond_truco(bot_call=True)
                    sleep(self.time*2.5)
            else:
                self.play_player()
            
            self.check_round(round)
            
            win_match,winner = self.win_match(summing_actual_points = True)
            if win_match:
                break
        
        cls()
        self.print_played_cards()
        if self.win_pc == True:
            print_slow(f'{self.bot.name} Gano la ronda')
        else:
            print_slow(f'Ganaste la ronda')
        sleep(self.time*2)
        if self.win_pc is False:
            self.actual_score[0] += self.truco
        elif self.win_pc is True:
            self.actual_score[1] += self.truco
        self.print_score()

    def play_player(self, only_cards = False):
        player_cards = self.cards[0]
        valid_inputs = []
        print('cartas disponibles'.center( self.center))
        self.print_card(cards = player_cards,center_left=False) 
        
        if only_cards==False:
            txt = ''
            for i in range(len(player_cards)):
                txt+= f'{i+1}: {player_cards[i]}  '
                valid_inputs.append(str(i+1))
            print_slow(txt)
            valid_calls = self.options_call()
            value = self.input_value(valid_inputs=valid_inputs+valid_calls)
            
            if value in ['1','2','3']:
                card_choice = player_cards[int(value)-1]
                self.cards_played['player'].append(card_choice)
                self.cards[0].remove(card_choice)
                self.play_pc = not self.play_pc
                
            elif value == 'M':
                self.mazo_call()
            elif value == 'E':
                valid_inputs = self.envido_call(refuse=True)
                value = self.input_value(valid_inputs=valid_inputs)
                if value in ['1','2','3']:
                    value = int(value)+1
                    print_slow(f'Jugador: {self.envido_options[value]}')  
                    sleep(self.time) 
                    responses_bot = self.bot.respond_envido(self.all_cards[1],at_least=value)
                    if responses_bot == 0:
                        self.print_phrase(self.bot.phrases['No Quiero'])
                        self.actual_score[0] = max(self.score_envido,1)
                        self.envido = 0
                    elif responses_bot == value:
                        self.print_phrase(self.bot.phrases['Quiero'])
                        self.envido = value
                        self.score_envido += self.envido_option_points[value]
                        self.calculate_envido()
                    else:
                        self.score_envido += self.envido_option_points[value]
                        self.respond_envido(responses_bot)
                    sleep(self.time*2.5) 

            elif value == 'T' and self.truco!=4:
                    print_slow(f'Le canto {self.truco_options[self.truco+1]} amigo')
                    responses_bot = self.bot.respond_truco(self.cards[1],self.cards_played,self.truco+1)
                    sleep(self.time)
                    if responses_bot == True:
                        self.truco += 1
                        self.respond_truco()
                    elif responses_bot == False:
                        self.truco += 1
                        self.quiero_pc = True
                        self.print_phrase(self.bot.phrases['Quiero'])       
                    else:
                        self.win_pc = False
                        self.end_round = True
                        self.print_phrase(self.bot.phrases['No Quiero'])
                    sleep(self.time*2.5)

    def respond_envido(self, envido):
        self.print_phrase(self.bot.phrases[self.envido_options[envido]])
        valid_inputs = self.envido_call(refuse=False,at_least=envido+1)
        value = self.input_value(valid_inputs=valid_inputs)
        
        if value == 'N':
            print_slow(f'Jugador: No quiero...')
            sleep(self.time)
            self.actual_score[1] = max(self.score_envido,1)
            self.envido = 0
        elif value == 'Q':
            print_slow(f'Jugador: Quiero...')
            self.score_envido += self.envido_option_points[envido]
            self.envido = envido
            self.calculate_envido()
        else:
            self.score_envido += self.envido_option_points[envido]
            self.envido = envido            
            value = int(value)+1
            print_slow(f'Jugador: {self.envido_options[value]}')  
            sleep(self.time) 
            responses_bot = self.bot.respond_envido(self.all_cards[1],at_least=value)
            if responses_bot == 0:
                self.print_phrase(self.bot.phrases['No Quiero'])
                self.actual_score[0] = max(self.score_envido,1)
                self.envido = 0
            elif responses_bot == value:
                self.print_phrase(self.bot.phrases['Quiero'])
                self.envido = value
                self.score_envido += self.envido_option_points[value]
                self.calculate_envido()
            else:
                self.score_envido += self.envido_option_points[responses_bot]
                self.respond_envido(responses_bot)
            sleep(self.time*2.5) 

    def respond_truco(self, bot_call = False):
        if bot_call:
            self.truco += 1
            self.print_phrase(self.bot.phrases[self.truco_options[self.truco]])
        else:
            try:
                self.print_phrase(self.bot.phrases[self.truco_options[self.truco+1]])
                self.truco += 1
            except:
                self.print_phrase(self.bot.phrases[self.truco_options[self.truco]])
            
        sleep(self.time*1.5)
        valid_inputs = self.truco_call(refuse=False)
        value = self.input_value(valid_inputs=valid_inputs)
        if value == 'Q':
            self.quiero_pc = False
            print_slow('Player: Quiero >:)')
            
        elif value == 'N':
            self.truco += -1
            self.mazo_call(txt='Player: No quiero...')
        if value == 'T':
            
            print_slow(f'Player: Le canto {self.truco_options[self.truco+1]} amigo')
            responses_bot = self.bot.respond_truco(self.cards[1],self.cards_played,self.truco+1)
            sleep(self.time)
            if responses_bot == True:
                self.truco += 1
                self.respond_truco()
            elif responses_bot == False:
                self.truco += 1
                self.quiero_pc = True
                self.print_phrase(self.bot.phrases['Quiero'])
            else:
                self.win_pc = False
                self.end_round = True
                self.print_phrase(self.bot.phrases['No Quiero'])

    #Options calls
    def options_call(self):
        txt = ''
        valid_inputs = []
        if self.envido not in [0,4]:
            if self.cards_played['player']:
                self.envido = 0
            else:
                txt += 'E: Envido, '
                valid_inputs.append('E')
        
        if self.truco != 4 and self.quiero_pc is not True:
            txt += f'T: {self.truco_options[self.truco+1].capitalize()}, '
            valid_inputs.append('T')

        txt += 'M: Mazo'
        valid_inputs.append('M')
        print_slow(txt)
        return valid_inputs
    
    def envido_call(self,at_least=1, refuse = False):
        txt = ''
        valid_inputs = []
        if at_least<=4:
            for i in self.envido_options:
                if i>=at_least:
                    txt += f'{i-1}: {self.envido_options[i]}, '
                    valid_inputs.append(str(i-1))
        if refuse:
            txt += f'V: volver'
            valid_inputs.append('V')
        else:
            txt += f'Q: Quiero, N: No Quiero'
            valid_inputs.append('Q')
            valid_inputs.append('N')
        print_slow(txt)
        return valid_inputs

    def truco_call(self,at_least=1, refuse = False):
        txt = ''
        valid_inputs = []
        if self.truco != 4:
            valid_inputs.append('T')
            txt = f'T: {self.truco_options[self.truco+1].capitalize()}, '
        if refuse:
            txt += f'V: volver'
            valid_inputs.append('V')
        else:
            txt += f'Q: Quiero, N: No Quiero'
            valid_inputs.append('Q')
            valid_inputs.append('N')
        print_slow(txt)
        return valid_inputs

    def mazo_call(self,txt = 'Player se ha ido al mazo'):
        self.end_round = True
        self.win_pc = True
        print_slow(txt) #MAZOWORD
        sleep(self.time)
    

    #responses
    def check_round(self,round):
        win_rounds,none = calculate_win(self.cards_played)
        response = ''
        winner = None
        player_text = f'has ganado la ronda!'
        pc_text = f'{self.bot.name} gano la rona...'
        if win_rounds['pc'] == win_rounds['player'] == win_rounds['draw'] == 1:
            if self.cards_played['pc'][0].value > self.cards_played['player'][0].value:
                winner = 1
            else:
                winner = 0
            self.end_round = True
        elif win_rounds['pc'] == 2 or (win_rounds['pc'] == win_rounds['draw'] == 1 and round != 3):
            winner = 1
            self.end_round = True
        elif win_rounds['player'] == 2 or (win_rounds['player'] == win_rounds['draw'] == 1 and round != 3):
            winner = 0
            self.end_round = True
        elif win_rounds['player'] == win_rounds['draw'] == 1 and round == 3:
            i=0
            if self.cards_played['pc'][0].value == self.cards_played['player'][0].value:
                i=1
                if self.cards_played['pc'][1].value == self.cards_played['player'][1].value:
                    i=2
            if self.cards_played['pc'][i].value > self.cards_played['player'][i].value:
                winner = 1
                self.end_round = True
            elif self.cards_played['pc'][i].value < self.cards_played['player'][i].value:
                winner = 0
                self.end_round = True
            elif self.start_pc:
                winner = 1
                self.end_round = True
            else:
                winner = 0
                self.end_round = True
        if winner == 0:
            self.win_pc = False
        elif winner == 1:
            self.win_pc = True  
    
    def win_match(self, summing_actual_points = False):
        score = [self.score[0],self.score[1]]
        if summing_actual_points:
            score[0] += self.actual_score[0]
            score[1] += self.actual_score[1]
        if score[0] >= self.objective:
            return True,0
        if score[1] >= self.objective:
            return True,1
        return False,None

    #input functions
    def input_value(self, valid_inputs = ['M']):
        while True:
            try:
                value:str = input('>> ')
                if value.upper() in valid_inputs:
                    return value.upper()
            except:
                pass
            print(f'(Ayuda) Elige una de estas opciones {valid_inputs}. Opcion elegida: {value}')

    #print functions
    def print_score(self,aument_score=True):
        if aument_score:
            self.score[0] += self.actual_score[0]
            self.score[1] += self.actual_score[1]
        txt = (len(self.bot.name)-2)*' '+'vs'
        cls()
        print('#'* self.center)
        print(f'{txt} - {self.bot.name}'.center(self.center))
        print(f'{self.score[0]:02d} - {self.score[1]:02d}'.center(self.center))
        print('#'* self.center)
        sleep(self.time*2)

    def print_card(self, cards = [], card = None, center_left = True):
        if card:
            extra = ' '*((self.center-len(frame))//2)
            print(extra+'o-------o')
            print(extra+f'|{card.number:02d}  {card.type_card.emoji} |')
            print(extra+f'|  {card.type_card.abbrevation}  |')
            print(extra+f'|{card.type_card.emoji}   {card.number:02d}|')
            print(extra+'o-------o')
        else: 
            frame = 'o-------o '* len(cards)
            top = ''
            middle = ''
            bottom = ''
            for i in range(len(cards)):
                top += f'|{cards[i].number:02d}   {cards[i].type_card.emoji}| '
                middle += f'|  {cards[i].type_card.abbrevation}  | '
                bottom += f'|{cards[i].type_card.emoji}   {cards[i].number:02d}| '

            if center_left:
                extra = ' '*((self.center-30)//2)
            else:
                extra = ' '*((self.center-len(frame))//2)
            print(extra+frame)
            print(extra+top)
            print(extra+middle)
            print(extra+bottom)
            print(extra+frame)

    def print_played_cards(self):
        print(f'Cartas Jugadas'.center( self.center))
        print(f'{self.bot.name}'.center( self.center))
        print('--------------------'.center( self.center))
        self.print_card(cards=self.cards_played['pc'])
        print('vs'.center( self.center))
        print('--------------------'.center( self.center))
        self.print_card(cards=self.cards_played['player'])
        print('--------------------'.center( self.center))

    def print_status(self):
        print(f'score: {self.score}')
        print(f'actual_score: {self.actual_score}')
        print(f'envido: {self.envido}')
        print(f'truco: {self.truco}')
        print(f'win_match: {self.win_match()}')
        print(f'win_match(summing_actual_points=True): {self.win_match(summing_actual_points=True)}')
        sleep(self.time*10)
    
    def print_intro_truco(self,print_dimensions:bool = True, time:int = 5):
        cls()

        if print_dimensions:
            print('DIMENSIONES RECOMENDADAS'.center(self.center))
            txt_horizontal = '1'
            for i in range(self.center-2):
                txt_horizontal+='-'
            txt_horizontal+=f'{self.center}'
            print_slow(txt_horizontal)
            for i in range(1,34):
                print_slow('|')
            print_slow('35')
            sleep(self.time*time)
        cls()
        print('+------------------------------------------+'.center(self.center))
        print('|                                          |'.center(self.center))
        print('| ######  ######  ##   ##   ####    ####   |'.center(self.center))
        print('|   ##    ##   ## ##   ##  ##     ##    ## |'.center(self.center))
        print('|   ##    ######  ##   ##  ##     ##    ## |'.center(self.center))
        print('|   ##    ##  ##  ##   ##  ##     ##    ## |'.center(self.center))
        print('|   ##    ##   ##  #####    ####    ####   |'.center(self.center))
        print('|                                          |'.center(self.center))
        print('+------------------------------------------+'.center(self.center))
        
        sleep(self.time*time)
    
    def print_phrase(self,phrases):
        phrase = choice(phrases)
        if type(phrase) == str:
            print_slow(f'{self.bot.name}: {phrase}')

        else:
            cont = 0
            len_phrase = len(phrase)
            for i in phrase:
                print_slow(f'{self.bot.name}: {i}')
                cont+=1
                if cont!=len_phrase:
                    sleep(self.time)
            
    def calculate_envido(self):
        sleep(self.time)
        cls()
        print()
        print('Envido:'.center( self.center))
        print('--------------------'.center( self.center))
        print(f'{self.bot.name}'.center( self.center))
        print()
        print('--------------------'.center( self.center))
        print(f'vs'.center( self.center))
        print()
        print('--------------------'.center( self.center))
        print()
        sleep(self.time*2)
        cls()
        print()
        print('Envido:'.center( self.center))
        print('--------------------'.center( self.center))
        print(f'{self.bot.name}'.center( self.center))
        #print(f'{self.cards_envido[1]}'.center( self.center))
        print()
        print('--------------------'.center( self.center))
        print(f'vs'.center( self.center))
        print(f'{self.cards_envido[0]}'.center( self.center))
        print('--------------------'.center( self.center))
        print()
        sleep(self.time*1.5)
        cls()
        print()
        print('Envido:'.center( self.center))
        print('--------------------'.center( self.center))
        print(f'{self.bot.name}'.center( self.center))
        print(f'{self.cards_envido[1]}'.center( self.center))
        print('--------------------'.center( self.center))
        print(f'vs'.center( self.center))
        print(f'{self.cards_envido[0]}'.center( self.center))
        print('--------------------'.center( self.center))
        print()
        sleep(self.time*1.5)

        if self.cards_envido[0] > self.cards_envido[1] or (self.cards_envido[0] == self.cards_envido[1] and self.start_pc == False) :
            if self.envido == 4:
                self.score_envido = self.objective-self.score[1]
            self.actual_score[0] = self.score_envido
            print_slow(f'Ganaste {self.score_envido} punto{"s" if self.score_envido!=1 else ""}!!')
        else:
            if self.envido == 4:
                self.score_envido = self.objective-self.score[0]
            self.actual_score[1] = self.score_envido
            print_slow(f'Perdiste... {self.score_envido} punto{"s" if self.score_envido!=1 else ""}... :(')
        self.envido = 0
        sleep(self.time*2.5)       



from pydantic import BaseModel
from typing import List, Optional

class TypeCardBase(BaseModel):
    name: str
    description: str
    emoji:str
    

class TypeCardInit(TypeCardBase):
    pass

class TypeCardGet(TypeCardBase):
    abbrevation:str




import os
import sys
import time

def print_slow(s, delay=0.01):
    if s:
        for c in s:
            sys.stdout.write(c)
            sys.stdout.flush()
            time.sleep(delay)
        print()
    
def cls():
    if os.name == 'nt':
        os.system('cls')
    # Unix/Linux/MacOS/BSD
    else:
        os.system('clear')


def clear_screen():
    # Windows
    if os.name == 'nt':
        os.system('cls')
    # Unix/Linux/MacOS/BSD
    else:
        os.system('clear')




from JMTrucoCmd.schemas.stats import StatsInit

class Stats():
    def __init__(self,stats_init:StatsInit) -> None:
        self.aggressive:int =  self.max_and_minum_init_stats(stats_init.aggressive)
        self.fisherman:int = self.max_and_minum_init_stats(stats_init.fisherman)
        self.liar:int = self.max_and_minum_init_stats(stats_init.liar)
        self.score_truco:int = stats_init.score_truco
        self.score_envido:int = stats_init.score_envido

    def __str__(self) -> str:
        return f'agresivo: {self.aggressive}, mentiroso: {self.liar}, pescador:{self.fisherman}'#, score_truco:{self.score_truco}, score_envido:{self.score_envido}'

    def max_and_minum_init_stats(self, n):
        if n<=0:
            return 1
        if n>=11:
            return 10
        return n


import setuptools

readme = open("./README.md","r")

setuptools.setup(
    name='JMTrucoCmd',
    packages=['JMTrucoCmd','JMTrucoCmd.models','JMTrucoCmd.schemas','JMTrucoCmd.functions'], # Mismo nombre que en la estructura de carpetas de arriba
    version='0.7.1',
    license='MIT', # La licencia que tenga tu paquete
    description='Esta es la descripcion del truco',
    long_description=readme.read(),
    long_description_content_type='text/markdown',
    author='Joaquin Morais',
    author_email='joaquinmoraislussary@gmail.com',
    url='https://github.com/JoaquinMorais/JMTrucoCmd', # Usa la URL del repositorio de GitHub
    download_url='https://github.com/JoaquinMorais/JMTrucoCmd/tarball/v0.7.1', # Te lo explico a continuación
    keywords=['testing','logging','example'], # Palabras que definan tu paquete
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    python_requires=">=3.6",
    install_requires=[
        'pydantic','typing'
    ]

)


from JMTrucoCmd.schemas.type_card import TypeCardInit, TypeCardGet

class TypeCard():
    def __init__(self,type_card_init:TypeCardInit) -> None:
        self.name:str = type_card_init.name.capitalize()
        self.abbrevation:str = type_card_init.name[0:3].upper()
        self.description:str = type_card_init.description
        self.emoji:str = type_card_init.emoji

    def __str__(self) -> str:
        return f'{self.name}'
    
    def get(self,completo=True) -> str:
        if completo:
            return f'{self.name}'
        return f'{self.abbrevation}'
    
    def json(self) -> TypeCardGet:
        return TypeCardGet(
            name=self.name,
            description=self.description,
            abbrevation=self.abbrevation
        )


#models
from JMTrucoCmd.models.bot import Bot
#schemas
from JMTrucoCmd.schemas.bot import BotInit
#data
from JMTrucoCmd.data.bots import bots_data

def get_bots(name=''):
    if name:
        return Bot(bots_data[name])
    bots = []
    for i in bots_data:
        bots.append(Bot(bots_data[i]))
    return bots